@@ -171,3 +171,7 @@ setMethod('summary', signature="MMSE", function(object, ..., silent=FALSE, Refs=
   invisible(df)
   
 })
+
+# ---- show method ----
+#' @rdname show-MSEtool
+setMethod("show", "MMSE", function(object) show_int(object))
