@@ -45,6 +45,9 @@
 #' @param SSB0x SSB0 for this simulation
 #' @param B0x B0 for this simulation
 #' @param MPA An array of spatial closures by year `[np,nf,nyears+proyears,nareas]`
+#' @param SRRfun A list of custom stock-recruit functions. List of length `np`
+#' @param SRRpars A list of custom stock-recruit parameters for simulation `x`. List of length `np`
+#' @param spawn_time_frac vector of relative spawn timing. Length nstock
 #' @author T.Carruthers
 #' @keywords internal
 popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, FretAx, Effind,
@@ -73,8 +76,10 @@ popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, Fre
   VBfind <- TEG(c(np, nf, n_age, 1, nareas))
   Nind <- TEG(c(np, n_age, 1, nareas))
   
-  FMx <- FMretx <- Fdist <- array(NA_real_, c(np, nf, n_age, nareas))
-  Find <- TEG(dim(Fdist))
+  FMx <- FMretx <- array(NA_real_, c(np, nf, n_age, nareas))
+  Fdist <- FMx
+  
+  Find <- TEG(dim(FMretx))
   
   VBcur <- array(NA, dim(VBfx)[-4]) # np x nfleet x nage x nareas
   Ecur <- array(NA, c(np, nf))
@@ -88,29 +93,95 @@ popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, Fre
     MPAthisyr <- abind::adrop(MPAthisyr, 3) # drop year dimension
     Nind[, 3] <- VBfind[, 4] <- y-1
     
-    #    p f a r                  p a y r                   p f a y
-    VBfx[VBfind] <- Bx[VBfind[, c(1,3,4,5)]] * VFx[VBfind[, 1:4]]
+    #    p f a r                  p a y r                 p f a y
+    VBfx[VBfind] <- Bx[VBfind[, c(1,3,4,5)]] * VFx[VBfind[, 1:4]] * qfracx[VBfind[, c(1,2)]]
+    
     VBcur[] <- VBfx[, , , y-1, ] # array(VBfx[,,,y-1,], c(np, nf, n_age, nareas))
     
     VBcur_a <- apply(VBcur, c(1,2,4), sum)
    
+    Ecur[] <- Effind[, , y-1] #matrix(Effind[, , y-1], np, nf)
+    Vcur[] <- VFx[, , , y-1] #array(VFx[, , , y-1], c(np, nf, n_age))
+    
     # Fdist[Find] <- VBcur[Find]*MPAthisyr[Find[,c(1,2,4)]]^Spat_targ[Find[, 1:2]]
+    
+    # optimize Spat_targ if neccesary - make sure overall F is correct
+    for (p in 1:np) {
+      # check if biomass equally distributed across areas
+      chk <- apply(VBcur_a[p,,, drop=FALSE], 3, mean)
+      if (chk[1] == mean(chk)) next()
+      
+      # optimize spat_targ 
+      par <- rep(0, nf) 
+      f_zero_fleet <- which(qfracx[p,]==0)
+      if (length(f_zero_fleet)>0)
+        par <- par[-f_zero_fleet] 
+      
+      N_area <- Nx[p,,y-1,]
+      M_age <- M_ageArrayx[p,,y-1]
+      if (length(par)==1) {
+        opt_spat_targ <- optimize(solve_spat_targ, 
+                               c(-3,3),
+                               pop=p, qsx=qsx, 
+                               qfracx=qfracx,
+                               Ecur=Ecur, n_age=n_age, nareas=nareas, 
+                               Fdist=Fdist, 
+                               Vcur=Vcur, 
+                               N_area=N_area, M_age=M_age,
+                               VBcur_a=VBcur_a,
+                               f_zero_fleet=f_zero_fleet,
+                               Asizex=Asizex)
+        
+        par <- opt_spat_targ$minimum
+        if (length(f_zero_fleet)>0) {
+          dummy <- rep(0, length(f_zero_fleet))
+          
+          temp <- rep(NA, length(c(par, f_zero_fleet)))
+          temp[f_zero_fleet] <- 0
+          temp[is.na(temp)] <- par
+          Spat_targ[p] <- temp
+        } else {
+          Spat_targ[p] <- opt_spat_targ$minimum
+        }
+      } else {
+        opt_spat_targ <- optim(par, 
+                               fn=solve_spat_targ,
+                               pop=p, qsx=qsx,  qfracx=qfracx,
+                               Ecur=Ecur, n_age=n_age, nareas=nareas, 
+                               Fdist=Fdist, 
+                               Vcur=Vcur, 
+                               N_area=N_area, M_age=M_age,
+                               VBcur_a=VBcur_a,
+                               f_zero_fleet=f_zero_fleet,
+                               Asizex=Asizex)
+        
+        par <- opt_spat_targ$par
+        if (length(f_zero_fleet)>0) {
+          dummy <- rep(0, length(f_zero_fleet))
+          
+          temp <- rep(NA, length(c(par, f_zero_fleet)))
+          temp[f_zero_fleet] <- 0
+          temp[is.na(temp)] <- par
+          Spat_targ[p,] <- temp
+        } else {
+          Spat_targ[p,] <- opt_spat_targ$par
+        }
+      }
+    }
+
     Fdist[Find] <- (VBcur_a[Find[,c(1,2,4)]]*MPAthisyr[Find[,c(1,2,4)]])^Spat_targ[Find[, 1:2]]
     Fdist[Find] <- Fdist[Find]/apply(Fdist,1:3,sum)[Find[,1:3]]
     Fdist[is.na(Fdist)] <- 0 # This is an NA catch for hermaphroditism
     
-    Ecur[] <- Effind[, , y-1] #matrix(Effind[, , y-1], np, nf)
-    Vcur[] <- VFx[, , , y-1] #array(VFx[, , , y-1], c(np, nf, n_age))
-    
     FMx[Find] <- qsx[Find[, 1]] * qfracx[Find[, 1:2]] * Ecur[Find[, 1:2]] * Fdist[Find] *
-      Vcur[Find[, 1:3]]/Asizex[Find[, c(1, 4)]]
+      Vcur[Find[, 1:3]]  / Asizex[Find[, c(1, 4)]]
     FMx[FMx > maxF] <- maxF # apply maxF
-    
+
     Retcur[] <- FretAx[, , , y-1] #array(FretAx[, , , 1], c(np, nf, n_age))
     FMretx[Find] <- qsx[Find[,1]] * qfracx[Find[,1:2]] * Ecur[Find[,1:2]] * Fdist[Find] *
-      Retcur[Find[, 1:3]]/Asizex[Find[, c(1, 4)]]
+      Retcur[Find[, 1:3]]  / Asizex[Find[, c(1, 4)]]
     FMretx[FMretx > maxF] <- maxF # apply maxF
-    
+
     # Update spawning biomass and recruitment for last year (calculated mid year if spawn_time_frac>0)
     ZMx_all <- array(NA, dim=c(np, n_age, nareas))
     Foverall <- apply(FMx, c(1,3,4), sum)
@@ -120,32 +191,31 @@ popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, Fre
     
     SSBx[Nind] <- Nx[Nind] * exp(-ZMx_all[Nind[,c(1,2,4)]] * spawn_time_frac[Nind[,1]]) * Fec_agex[Nind[, 1:3]]
     SSNx[Nind] <- Nx[Nind] * exp(-ZMx_all[Nind[,c(1,2,4)]] * spawn_time_frac[Nind[,1]]) * Mat_agex[Nind[, 1:3]]
-  
-    # Calculate SSB for S-R relationship
-    SSB_SR <- local({
-      SSBtemp <- array(NA_real_, dim(SSBx[,,y-1,])) # np x n_age x nareas
-      SSBtemp[] <- SSBx[,,y-1,, drop=FALSE]
-      if (length(SexPars$SSBfrom)) { # use SSB from another stock to predict recruitment
-        sapply(1:np, function(p) apply(SexPars$SSBfrom[p, ] * SSBtemp, 2:3, sum), simplify = "array") %>%
-          aperm(c(3, 1, 2))
-      } else {
-        SSBtemp
-      }
-    })
  
     # Recruitment for last year
     if (y>2) {
+      # Calculate SSB for S-R relationship
+      SSB_SR <- local({
+        SSBtemp <- array(NA_real_, dim(SSBx[,,y-1,])) # np x n_age x nareas
+        SSBtemp[] <- SSBx[,,y-1,, drop=FALSE]
+        if (length(SexPars$SSBfrom)) { # use SSB from another stock to predict recruitment
+          sapply(1:np, function(p) apply(SexPars$SSBfrom[p, ] * SSBtemp, 2:3, sum), simplify = "array") %>%
+            aperm(c(3, 1, 2))
+        } else {
+          SSBtemp
+        }
+      })
       if (length(dim(SSB_SR))==2) {
         Nx[, 1, y-1, ] <- sapply(1:np, function(p) {
           calcRecruitment_int(SRrel = SRrelx[p], SSBcurr = SSB_SR, recdev = Perrx[p, y-1+n_age-1], hs = hsx[p], 
                               aR = aRx[p, 1], bR = 1/sum(1/bRx[p, ]), R0a = R0ax[p, ], SSBpR = SSBpRx[p, 1],
-                              SRRfun, SRRpars)
+                              SRRfun[[p]], SRRpars[[p]])
         }) %>% t()
       } else {
         Nx[, 1, y-1, ] <- sapply(1:np, function(p) {
           calcRecruitment_int(SRrel = SRrelx[p], SSBcurr = SSB_SR[p, , ], recdev = Perrx[p, y-1+n_age-1], hs = hsx[p], 
                               aR = aRx[p, 1], bR = 1/sum(1/bRx[p, ]), R0a = R0ax[p, ], SSBpR = SSBpRx[p, 1],
-                              SRRfun, SRRpars)
+                              SRRfun[[p]], SRRpars[[p]])
         }) %>% t()
       }
    
@@ -153,6 +223,9 @@ popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, Fre
       Bx[, , y-1, ] <- Nx[, , y-1, ] * replicate(nareas,WatAgex[,,y-1])
     }
     
+    SexPars_y <- SexPars
+    SexPars_y$Herm <- subsetHerm(SexPars_y$Herm, y = y)
+    
     out <- popdynOneMICE(np, nf, nareas, maxage,
                          Ncur = array(Nx[, , y-1, ], dim(Nx)[c(1:2, 4)]),
                          Bcur = array(Bx[, , y-1, ], dim(Nx)[c(1:2, 4)]),
@@ -173,11 +246,11 @@ popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, Fre
                          t0x = t0arrayx[, y],
                          Mx = Marrayx[, y-1],
                          R0x = R0x, R0ax = R0ax, SSBpRx = SSBpRx, ax = ay[, y],
-                         bx = by[, y], Rel = Rel, SexPars = SexPars, x = x,
+                         bx = by[, y], Rel = Rel, SexPars = SexPars_y, x = x,
                          plusgroup = plusgroup, SSB0x = SSB0x, B0x = B0x,
                          Len_agenext = array(Len_agex[, , y], c(np, n_age)),
                          Wt_agenext = array(WatAgex[, , y], c(np, n_age)),
-                         SRRfun=SRRfun, SRRpars=SRRpars)
+                         SRRfun=SRRfun, SRRpars=SRRpars, ycur = y-1)
     
     # update arrays 
     if (y <= nyears) {
@@ -235,7 +308,8 @@ popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, Fre
        Zx=Zx, #18
        Fty=Fty, #19
        M_ageArrayx=M_ageArrayx, #20
-       Fec_Agex=Fec_agex) #21
+       Fec_Agex=Fec_agex, # 21
+       Spat_targ=Spat_targ) #22
 }
 
 
@@ -289,6 +363,7 @@ popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, Fre
 #' @param Wt_agenext Matrix `[stock, age]` of next year's weight-at-age
 #' @param SRRfun  Optional. A stock-recruit function used if `SRrelc =3` 
 #' @param SRRpars Optional. A named list of arguments for `SRRfun`
+#' @param ycur Integer. The current year of the simulation
 #' @author T.Carruthers
 #' @keywords internal
 popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMretx, FMx, PerrYrp,
@@ -298,15 +373,11 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
                           Kx, Linfx, t0x, Mx, R0x, R0ax, SSBpRx, ax, bx, Rel, SexPars, x,
                           plusgroup, SSB0x, B0x,
                           Len_agenext, Wt_agenext,
-                          SRRfun, SRRpars) {
+                          SRRfun, SRRpars, ycur) {
   
   n_age <- maxage + 1
   Nind <- TEG(dim(Ncur)) # p, age, area
   
-  # Survival this year
-  surv <- array(c(rep(1,np), t(exp(-apply(M_agecur, 1, cumsum)))[, 1:(n_age-1)]), c(np, n_age))  # Survival array
-  surv[plusgroup, n_age] <- surv[plusgroup, n_age]/(1 - exp(-M_agecur[plusgroup, n_age])) # plusgroup
-  
   # These could change, these are values previous to MICE rel
   oldMx <- Mx # M of mature animals
   oldM_agecur <- M_agecur
@@ -315,48 +386,73 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
   oldWt_agenext <- Wt_agenext
   oldFec_agenext <- Fec_agenext
   
-  if (length(Rel)) { # MICE relationships, parameters that could change: M, K, Linf, t0, a, b, hs
-    Responses <- ResFromRel(Rel, Bcur, SSBcur, Ncur, SSB0x, B0x, seed = 1, x)
-    DV <- sapply(Responses, function(xx) xx[4])
+  oldPerrYrp <- PerrYrp
+  
+  if (length(Rel)) { # MICE relationships, parameters that could change: M, K, Linf, t0, a, b, hs, Perr_y
+    Dlag_all <- sapply(Rel, get_Dlag)
     
-    for (r in 1:length(Responses)) { # e.g., Mx[1] <- 0.4 - operations are sequential
-      eval(parse(text = paste0(DV[r], "[", Responses[[r]][3], "]<-", Responses[[r]][1])))
-    }
-    
-    if (any(DV %in% c("Linfx", "Kx", "t0x"))) { # only update Len_age for next year if MICE response is used
-      Len_agenext <- matrix(Linfx * (1 - exp(-Kx * (rep(0:maxage, each = np) - t0x))), nrow = np)
-      Len_agenext[Len_agenext < 0] <- tiny
-    } 
-    if (any(DV %in% c("Linfx", "Kx", "t0x", "ax", "bx"))) { # only update Len_age/Wt_age for next year if MICE response
-      # update relative fecundity-at-age for SSB
-      Fec_per_weight <- array(NA, dim = dim(Fec_agenext))
-      Fec_per_weight[Nind[, 1:2]] <- Fec_agenext[Nind[, 1:2]]/Wt_agenext[Nind[ ,1:2]]
+    if (any(Dlag_all == "current")) {  # Proceed except if Rel[[r]]$Dlag = "next"
+      Responses <- ResFromRel(Rel[Dlag_all == "current"], Bcur, SSBcur, Ncur, SSB0x, B0x, seed = 1, x, y = ycur)
       
-      Wt_agenext <- ax * Len_agenext ^ bx # New weight-at-age
-      Fec_agenext[Nind[, 1:2]] <- Fec_per_weight[Nind[, 1:2]] * Wt_agenext[Nind[, 1:2]]
+      Dmodnam <- sapply(Responses, getElement, "modnam")
+      Dp <- sapply(Responses, getElement, "Dp")
+      Dval <- lapply(Responses, getElement, "value")
+      Dage <- lapply(Responses, getElement, "age")
+      Dmult <- sapply(Responses, getElement, "mult")
+      
+      Rel_txt <- sapply(1:length(Responses), function(r) {
+        if (Dmodnam[r] == "Mx" && all(!is.na(Dage[[r]]))) { # Age-specific M
+          Rel_var <- paste0("M_agecur[", Dp[r], ", Dage[[r]] + 1]")
+        } else {
+          Rel_var <- paste0(Dmodnam[r], "[", Dp[r], "]")
+        }
+        if (Dmult[r]) {
+          Rel_val <- paste("Dval[[r]] *", Rel_var)
+        } else {
+          Rel_val <- "Dval[[r]]"
+        }
+        paste(Rel_var, "<-", Rel_val)
+      })
+      
+      for (r in 1:length(Responses)) { # e.g., Mx[1] <- 0.4 - operations are sequential
+        eval(parse(text = Rel_txt[r]))
+      }
+      
+      if (any(Dmodnam %in% c("Linfx", "Kx", "t0x"))) { # only update Len_age for next year if MICE response is used
+        Len_agenext <- matrix(Linfx * (1 - exp(-Kx * (rep(0:maxage, each = np) - t0x))), nrow = np)
+        Len_agenext[Len_agenext < 0] <- tiny
+      } 
+      if (any(Dmodnam %in% c("Linfx", "Kx", "t0x", "ax", "bx"))) { # only update Len_age/Wt_age for next year if MICE response
+        # update relative fecundity-at-age for SSB
+        Fec_per_weight <- array(NA, dim = dim(Fec_agenext))
+        Fec_per_weight[Nind[, 1:2]] <- Fec_agenext[Nind[, 1:2]]/Wt_agenext[Nind[ ,1:2]]
+        
+        Wt_agenext <- ax * Len_agenext ^ bx # New weight-at-age
+        Fec_agenext[Nind[, 1:2]] <- Fec_per_weight[Nind[, 1:2]] * Wt_agenext[Nind[, 1:2]]
+      }
+      
+      # Recalc M_age for this year (only if age-invariant M was updated) ------------------
+      if (any(Dmodnam == "Mx") && all(M_agecur == oldM_agecur)) {
+        M_agecur <- oldM_agecur * Mx/oldMx
+      }
+      
+      # --- This is redundant code for updating parameters when R0 changes -----
+      # surv <- cbind(rep(1,np),t(exp(-apply(M_agecur, 1, cumsum)))[, 1:(maxage-1)])  # Survival array
+      # SSB0x<-apply(R0x*surv*Mat_agecur*Wt_age,1,sum)
+      #SSBpRx<-SSB0x/R0x
+      #SSBpRax<-SSBpRx*distx
+      #SSB0ax<-distx*SSB0x
+      #R0ax<-distx*R0x
+      #R0recalc thus aR bR recalc ---------------
+      #bRx <- matrix(log(5 * hsx)/(0.8 * SSB0ax), nrow=np)  # Ricker SR params
+      #aRx <- matrix(exp(bRx * SSB0ax)/SSBpRx, nrow=np)  # Ricker SR params
     }
-    
-    # Recalc M_age for this year ------------------
-    if (any(DV == "Mx")) {
-      M_agecur <- M_agecur * Mx/oldMx
-      surv <- array(c(rep(1,np), t(exp(-apply(M_agecur, 1, cumsum)))[, 1:(n_age-1)]), 
-                    c(np, n_age))
-      surv[plusgroup, n_age] <- surv[plusgroup, n_age]/(1 - exp(-M_agecur[plusgroup, n_age])) # plusgroup
-    }
-    
-    # --- This is redundant code for updating parameters when R0 changes -----
-    # surv <- cbind(rep(1,np),t(exp(-apply(M_agecur, 1, cumsum)))[, 1:(maxage-1)])  # Survival array
-    # SSB0x<-apply(R0x*surv*Mat_agecur*Wt_age,1,sum)
-    #SSBpRx<-SSB0x/R0x
-    #SSBpRax<-SSBpRx*distx
-    #SSB0ax<-distx*SSB0x
-    #R0ax<-distx*R0x
-    #R0recalc thus aR bR recalc ---------------
-    #bRx <- matrix(log(5 * hsx)/(0.8 * SSB0ax), nrow=np)  # Ricker SR params
-    #aRx <- matrix(exp(bRx * SSB0ax)/SSBpRx, nrow=np)  # Ricker SR params
-    
   } # end of MICE
   
+  # Survival this year
+  surv <- array(c(rep(1,np), t(exp(-apply(M_agecur, 1, cumsum)))[, 1:(n_age-1)]), c(np, n_age))  # Survival array
+  surv[plusgroup, n_age] <- surv[plusgroup, n_age]/(1 - exp(-M_agecur[plusgroup, n_age])) # plusgroup
+  
   # Vulnerable biomass calculation (current year) -------------
   VBft <- Fdist <- array(NA, c(np, nf, n_age, nareas))
   VBind <- TEG(dim(VBft))
@@ -384,8 +480,8 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
     zero.m.ind <- which(M_agecur[p,]==0)
     if (length(zero.m.ind)>0) {
       # set N above maxage for this stock to zero
-      Nnext[p,min(zero.m.ind)-1,] <- Nnext[p,min(zero.m.ind)-1,] + apply(Nnext[p,zero.m.ind,], 2, sum)
-      Nnext[p,zero.m.ind,] <- 0 
+      Nnext[p, min(zero.m.ind)-1, ] <- Nnext[p, min(zero.m.ind)-1, ] + apply(Nnext[p, zero.m.ind, , drop = FALSE], 3, sum)
+      Nnext[p, zero.m.ind, ] <- 0 
     }
   }
   
@@ -396,7 +492,7 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
       ps <- as.numeric(strsplit(names(SexPars$Herm)[i], "_")[[1]][2:3])
       pfrom <- ps[2]
       pto <- ps[1]
-      frac <- rep(1,maxage)
+      frac <- rep(1, n_age)
       frac[1:length(SexPars$Herm[[i]][x, ])] <- SexPars$Herm[[i]][x, ]
       h_rate <- hrate(frac)
       Nnext[pto, , ] <- Nnext[pto, , ] * (frac > 0) # remove any recruitment
@@ -407,11 +503,9 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
   }
   
   # Calculate SSB for S-R relationship
+  SSBtemp <- array(NA_real_, dim(Nnext)) # np x n_age x nareas
+  SSBtemp[Nind] <- Nnext[Nind] * Fec_agenext[Nind[, 1:2]]
   SSB_SR <- local({
-    SSBtemp <- array(NA_real_, dim(Nnext)) # np x n_age x nareas
-
-    SSBtemp[Nind] <- Nnext[Nind] * Fec_agenext[Nind[, 1:2]]
-
     if (length(SexPars$SSBfrom)) { # use SSB from another stock to predict recruitment
       sapply(1:np, function(p) apply(SexPars$SSBfrom[p, ] * SSBtemp, 2:3, sum), simplify = "array") %>%
         aperm(c(3, 1, 2))
@@ -419,12 +513,40 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
       SSBtemp
     }
   })
+  
+  # Re-run MICE if any of them use recruitment deviations with next year's abundance/biomass
+  if (length(Rel) && any(Dlag_all == "next")) {
+    Responses2 <- local({
+      Bnext <- SSBnext <- array(NA_real_, dim(Nnext)) # np x n_age x nareas
+      Bnext[Nind] <- Nnext[Nind] * Wt_agenext[Nind[, 1:2]]
+      ResFromRel(Rel[Dlag_all == "next"], Bcur = Bnext, SSBcur = SSBtemp, Ncur = Nnext, SSB0x, B0x, seed = 1, x, y = ycur)
+    })
+    
+    Dmodnam2 <- sapply(Responses2, getElement, "modnam")
+    Dp2 <- sapply(Responses2, getElement, "Dp")
+    Dval2 <- lapply(Responses2, getElement, "value")
+    Dmult2 <- sapply(Responses2, getElement, "mult")
+    
+    Rel_txt2 <- sapply(1:length(Responses2), function(r) {
+      Rel_var2 <- paste0(Dmodnam2[r], "[", Dp2[r], "]")
+      if (Dmult2[r]) {
+        Rel_val2 <- paste("Dval2[r] *", Rel_var2)
+      } else {
+        Rel_val2 <- "Dval2[[r]]"
+      }
+      paste(Rel_var2, "<-", Rel_val2)
+    })
+    
+    for (r in 1:length(Responses2)) { # e.g., PerrYrp[1] <- 1.5 - operations are sequential
+      eval(parse(text = Rel_txt2[r]))
+    }
+  }
 
   # Generate next year's recruitment (this will get updated if spawn_time_frac > 0 )
   Nnext[, 1, ] <- sapply(1:np, function(p) {
     calcRecruitment_int(SRrel = SRrelx[p], SSBcurr = SSB_SR[p, , ], recdev = PerrYrp[p], hs = hsx[p],
                         aR = aRx[p, 1], bR = 1/sum(1/bRx[p, ]), R0a = R0ax[p, ], SSBpR = SSBpRx[p, 1],
-                        SRRfun, SRRpars)
+                        SRRfun[[p]], SRRpars[[p]])
   }) %>% t()
 
   # Movement
@@ -465,7 +587,8 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
        Bnext=Bnext, #23
        SSNnext=SSNnext, #24
        SSBnext=SSBnext,#25
-       Fec_agenext=Fec_agenext) #26
+       Fec_agenext=Fec_agenext,#26
+       PerrYrp=PerrYrp)#27
   
 }
 
@@ -483,47 +606,112 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
 #' @param B0 A vector of unfished biomass `[stock]`
 #' @param seed Seed for sampling.
 #' @param x The simulation number.
-#' @author T.Carruthers
+#' @param y The current year of the simulation.
+#' @author T. Carruthers, Q. Huynh
 #' @keywords internal
-ResFromRel <- function(Rel, Bcur, SSBcur, Ncur, SSB0, B0, seed, x) {
+ResFromRel <- function(Rel, Bcur, SSBcur, Ncur, SSB0, B0, seed, x, y) {
 
-  IVnams <- c("B", "SSB", "N", "SSB0", "B0", "x")
-  IVcode <- c("Bcur", "SSBcur", "Ncur", "SSB0", "B0", "x")
+  IVnams <- c("B", "SSB", "N", "Nage", "SSB0", "B0", "x", "y")           # Variable in MICE function
+  IVcode <- c("Bcur", "SSBcur", "Ncur", "Nage", "SSB0", "B0", "x", "y")  # Variable in OM
   B <- apply(Bcur, 1, sum)
   SSB <- apply(SSBcur, 1, sum)
   N <- apply(Ncur, 1, sum)
+  Nage <- apply(Ncur, 1:2, sum)
 
-  DVnam <- c("M", "a", "b", "R0", "hs", "K", "Linf", "t0")
-  modnam <- c("Mx", "ax", "bx", "R0x", "hsx", "Kx", "Linfx", "t0x")
+  DVnam <- c("M", "a", "b", "R0", "hs", "K", "Linf", "t0", "Perr_y")            # Variable in MICE function
+  modnam <- c("Mx", "ax", "bx", "R0x", "hsx", "Kx", "Linfx", "t0x", "PerrYrp")  # Variable in OM
 
   nRel <- length(Rel)
   
   out <- lapply(1:nRel, function(r) {
     fnams <- names(Rel[[r]]$model)
     DV <- fnams[1]
-    Dp <- unlist(strsplit(DV, "_"))[2]
-    Dnam <- unlist(strsplit(DV, "_"))[1]
+    Dp <- get_Dp(DV)
+    Dnam <- get_Dnam(DV)
     IV <- fnams[-1]
     nIV <- length(IV)
     
-    newdata <- sapply(IV, function(iv, B, SSB, N, SSB0, B0, x)  {
+    newdata <- lapply(IV, function(iv, B, SSB, N, Nage, SSB0, B0, x, y)  {
       IVs <- unlist(strsplit(iv, "_"))
       p <- ifelse(length(IVs) == 1, 1, as.numeric(IVs[2]))
-      get(IVs[1], inherits = FALSE)[p] # Get independent variables from OM
-    }, B = B, SSB = SSB, N = N, SSB0 = SSB0, B0 = B0, x = x) %>% 
-      matrix(nrow = 1) %>% as.data.frame() %>% structure(names = IV)
+      
+      # Get independent variables from OM
+      OMvar <- get(IVs[1], inherits = FALSE)
+      if (is.matrix(OMvar)) {
+        OMvar[p, ]
+      } else {
+        OMvar[p]
+      }
+    }, B = B, SSB = SSB, N = N, Nage = Nage, SSB0 = SSB0, B0 = B0, x = x, y = y) %>% 
+      structure(names = IV) %>%
+      unlist()
     
-    ys <- predict(Rel[[r]], newdata = newdata)
+    # If IV = Nage_p, then name will be Nage_pa (population, age)
+    newdata2 <- matrix(newdata, nrow = 1) %>%
+      as.data.frame() %>% 
+      structure(names = names(newdata))
+    
+    ys <- predict(Rel[[r]], newdata = newdata2)
     templm <- Rel[[r]]
     templm$fitted.values <- ys
     ysamp <- stats::simulate(templm, nsim = 1, seed = seed) %>% unlist()
     
-    c(ysamp, DV, Dp, modnam[match(Dnam, DVnam)])
+    rel_r <- list(
+      value = ysamp,
+      DV = DV,
+      Dp = Dp,
+      modnam = modnam[match(Dnam, DVnam)]
+    )
+    
+    if (Dnam == "Perr_y") {
+      if (!is.null(Rel[[r]]$lag)) {
+        lag <- Rel[[r]]$lag # choices are "current" or "next"
+      } else {
+        lag <- "current"
+      }
+      if (lag != "next") lag <- "current"
+    } else {
+      lag <- NA_character_
+    }
+    rel_r[["lag"]] <- lag
+    
+    if (Dnam == "M") {
+      if (!is.null(Rel[[r]]$age)) {
+        age <- Rel[[r]]$age
+      } else {
+        age <- NA_integer_
+      }
+    } else {
+      age <- NA_integer_
+    }
+    rel_r[["age"]] <- age
+    
+    rel_r[["mult"]] <- FALSE
+    if (!is.null(Rel[[r]]$mult) && is.logical(Rel[[r]]$mult)) rel_r[["mult"]] <- Rel[[r]]$mult
+    
+    return(rel_r)
   })
-  out
+  
+  return(out)
+}
+
+get_Dlag <- function(Rel) {
+  lag <- Rel$lag
+  if (is.null(lag) || lag != "next") lag <- "current"
+  return(lag)
 }
 
 
+get_Dp <- function(DV) {
+  x <- unlist(strsplit(DV, "_"))
+  x[length(x)]
+}
+
+get_Dnam <- function(DV) {
+  x <- unlist(strsplit(DV, "_"))
+  paste0(x[-length(x)], collapse = "_")
+}
+  
 #' Derives the rate of exchange from one sex to another based on asymptotic fraction
 #'
 #' @param frac A vector of asymptotic sex fraction (must start with zero and end with 1)
